from .validator import Validator


class Number(Validator):

    def __init__(self, error="Value must be a number."):
        self.processors = []
        self.processors.append({
            'action': self.__assert_number,
            'attribs': {
                'error': error
            }
        })

    def __assert_number(self, data, attribs):
        if data is None:
            return self.validation_success(data)

        if type(data) not in (float, int, str):
            return self.validation_error(data, attribs['error'])

        try:
            data = float(data)
            return self.validation_success(data)
        except ValueError:
            return self.validation_error(data, attribs['error'])

    def integer(self):
        self.processors.append({
            'action': self.__convert_to_int,
            'attribs': None
        })

        return self

    def __convert_to_int(self, data, attribs):
        return self.validation_success(data) if data is None else self.validation_success(int(data))

    def required(self, error="Value is required"):
        self.processors.append({
            'action': self.__assert_required,
            'attribs': {
                'error': error
            }
        })

        return self

    def __assert_required(self, data, attribs):
        return self.validation_error(data, attribs.get('error')) if data is None else self.validation_success(data)

    def between(self, start, end, error=None):
        type_of_start = type(start)
        type_of_end = type(end)

        if type_of_start is not float and type_of_start is not int:
            raise ValueError('Range start must be a number.')

        if type_of_end is not float and type_of_end is not int:
            raise ValueError('Range end must be a number.')

        if error is None:
            error = f"Value must be between {start} and {end}"

        self.processors.append({
            'action': self.__assert_between,
            'attribs': {
                'start': start,
                'end': end,
                'error': error
            }
        })

        return self

    def __assert_between(self, data, attribs):
        if data is None or (data >= attribs.get('start') and data <= attribs.get('end')):
            return self.validation_success(data)
        else:
            return self.validation_error(data, attribs.get('error'))

    def max(self, max_value, error=None):
        type_of_max_value = type(max_value)
        if type_of_max_value is not int and type_of_max_value is not float:
            raise ValueError('max_value must be a number.')

        if error is None:
            error = f"Maximum value allowed is {max_value}"

        self.processors.append({
            'action': self.__assert_max,
            'attribs': {
                'max_value': max_value,
                'error': error
            }
        })

        return self

    def __assert_max(self, data, attribs):
        if data is None or data <= attribs.get('max_value'):
            return self.validation_success(data)
        else:
            return self.validation_error(data, attribs.get('error'))

    def min(self, min_value, error=None):
        type_of_min_value = type(min_value)
        if type_of_min_value is not int and type_of_min_value is not float:
            raise ValueError('min_value must be a number.')

        if error is None:
            error = f"Minimum value required is {min_value}"

        self.processors.append({
            'action': self.__assert_min,
            'attribs': {
                'min_value': min_value,
                'error': error
            }
        })

        return self

    def __assert_min(self, data, attribs):
        if data is None or data >= attribs.get('min_value'):
            return self.validation_success(data)
        else:
            return self.validation_error(data, attribs.get('error'))



class Validator(object):
    def __init__(self):
        self.processors = []

    def validation_error(self, data, error):
        return {
            'success': False,
            'data': data,
            'error': error
        }

    def validation_success(self, data):
        return {
            'success': True,
            'data': data,
            'error': None
        }

    def validate(self, data):
        last_processor_result = {
            'data' : data
        }

        for processor in self.processors:
            last_processor_result = processor['action'](
                last_processor_result['data'], processor['attribs'])

            if last_processor_result['success'] is False:
                break

        if last_processor_result['success'] is False:
            return self.validation_error(data,last_processor_result['error'])
        else:
            return last_processor_result



from setuptools import setup,find_packages

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name="kanpai",
    version="0.1.1",
    keywords="schema json validation request-validation input-validation",
    author="Chandrakanta Pal",
    author_email="pal.chandrakanta@gmail.com",
    description="Python JSON schema validator with better error message",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://gitlab.com/chandrakantap/kanpai.git",
    packages=find_packages(exclude=("tests",)),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)



from .validator import Validator


class String(Validator):

    def __assert_string(self, data, attribs):
        if data is None:
            return self.validation_success(data)

        data_type = type(data)
        if data_type is int or data_type is float or data_type is str:
            return self.validation_success(str(data))
        else:
            return self.validation_error(data, attribs['error'])

    def __init__(self, error='Value must be a string'):
        self.processors = []
        self.processors.append({
            'action': self.__assert_string,
            'attribs': {
                'error': error
            }
        })

    def __assert_required(self, data, attribs):
        if data is None or len(data) is 0:
            return self.validation_error(data, attribs['error'])
        else:
            return self.validation_success(data)

    def required(self, error='Value is required'):
        self.processors.append({
            'action': self.__assert_required,
            'attribs': {
                'error': error
            }
        })
        return self

    def __assert_max(self, data, attribs):
        if data is not None and len(data) > attribs['max_length']:
            return self.validation_error(data, attribs['error'])
        else:
            return self.validation_success(data)

    def max(self, max_length, error=None):
        if type(max_length) is not int:
            raise ValueError(
                'value for max_length is expected to be an integer')

        if error is None:
            error = f"Maximum length allowed is {max_length}"

        self.processors.append({
            'action': self.__assert_max,
            'attribs': {
                'max_length': max_length,
                'error': error
            }
        })

        return self

    def __trim(self, data, attribs):
        return self.validation_success(data.strip() if data is not None else None)

    def trim(self):
        self.processors.append({
            'action': self.__trim,
            'attribs': None
        })
        return self



from .validator import Validator


class Object(Validator):

    def __init__(self, schema, error="Expecting an object.", ignore_extra_key=False):
        if type(schema) is not dict:
            raise ValueError('Expecting schema to be a dictionary.')

        for key, validator in schema.items():
            if not isinstance(validator, Validator):
                raise TypeError(f'Expecting a instance of validator in {key}')

        self.processors = []
        self.processors.append({
            'action': self.__assert_data_schema,
            'attribs': {
                'schema':schema,
                'error': error,
                'ignore_extra_key':ignore_extra_key
            }
        })

    def required(self,error='Value is required'):
        self.processors.append({
            'action': self.__assert_required,
            'attribs': {
                'error': error
            }
        })

        return self

    def __assert_required(self,data,attribs):
        if data is None:
            return self.validation_error(data,attribs.get('error'))
        else:
            return self.validation_success(data)

    def __assert_data_schema(self,data,attribs):
        if data is None:
            return self.validation_success(data)

        if type(data) is not dict:
            return self.validation_error(data, {'data': attribs.get('error')})

        validation_success = True
        validation_data = data
        validation_error = {}

        schema = attribs.get('schema',{})

        for key,validator in schema.items():
          validation_result = validator.validate(data.get(key))
          validation_success = validation_success and validation_result['success']
          validation_data[key] = validation_result['data']
          if validation_result['success'] is False:
            validation_error[key] = validation_result['error']

        
        if attribs.get('ignore_extra_key') is False:
          for key,value in data.items():
            if schema.get(key) is None:
              validation_success = False
              validation_error[key] = f"Unexpected {key}"

        if validation_success is False:
          return self.validation_error(data,validation_error)
        else:
          return self.validation_success(validation_data)


